/*
 * parse_conf.c
 * Routines to parse the (XML) bgrotd conf file.
 * bgrot project
 * REQUIRES: libxml2
 * $Fuller: myprograms/bgrot/common/parse/parse_conf.c,v 1.9 2003/03/23 14:53:42 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libxml/parser.h>

#include <bgrot.h>
#define PARSE_CONF
#include <parse_macros.h>
#ifdef BGROTD
#include <bgrotd.h>
#endif


/* parse_conf: Parse out the bgrotd conf file */
BGROTD_CONF *
parse_conf(const char *conffile)
{
	xmlDocPtr parsed;
	xmlNodePtr cur;
	BGROTD_CONF *bgconf;
	char *vtmp;

	/* Warm up */
	bgconf = alloc_bgrotd_conf();
	if(bgconf==NULL)
		BAIL("alloc_bgrotd_conf");

	/* Parse the file */
	parsed = xmlParseFile(conffile);
	if(parsed==NULL)
		BAIL("xmlParseFile")

	/* Start picking through the parsed file */
	cur = xmlDocGetRootElement(parsed);
	if(cur==NULL)
		BAIL("xmlDocGetRootElement");

	/* Top level should be <bgrotdconf> node */
	if(xmlStrcmp(cur->name, (const xmlChar *) "bgrotdconf"))
		BAIL("Should be <bgrotdconf>!")
	vtmp = xmlGetProp(cur, (const xmlChar *) "version");
	if(vtmp==NULL)
		BAIL("Can't get version")
	bgconf->version = (float)atof(vtmp);
	free(vtmp);

	/* Start our big recursion */
	cur = cur->xmlChildrenNode;
	while(cur != NULL)
	{
		if( !(strcmp(cur->name, "basedir")) )
		{
			bgconf->basedir =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( !(strcmp(cur->name, "rundir")) )
		{
			bgconf->rundir =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( !(strcmp(cur->name, "listdir")) )
		{
			bgconf->listdir =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( !(strcmp(cur->name, "list")) )
		{
			vtmp = xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
			bgconf->sel_list_id = atoi(vtmp);
			free(vtmp);
		}
		else if( !(strcmp(cur->name, "int")) )
		{
			if(bgconf->conf_override==NULL)
			{
				bgconf->conf_override = alloc_ilconf();
				if(bgconf->conf_override==NULL)
					BAIL("Can't alloc for bgconf->conf_override");
			}
			vtmp = xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
			bgconf->conf_override->interval = atoi(vtmp);
			free(vtmp);

			vtmp = xmlGetProp(cur, "vary");
			bgconf->conf_override->rvary = atoi(vtmp);
			free(vtmp);
		}
		else if( !(strcmp(cur->name, "display")) )
		{
			if(bgconf->conf_override==NULL)
			{
				bgconf->conf_override = alloc_ilconf();
				if(bgconf->conf_override==NULL)
					BAIL("Can't alloc for bgconf->conf_override");
			}
			bgconf->conf_override->display =
					xmlNodeListGetString(parsed, cur->xmlChildrenNode, 1);
		}
		else if( cur->type == XML_COMMENT_NODE )
		{
			/* Comment */
		}
		else if( !(xmlIsBlankNode(cur)) )
		{
			char bgerr[255];
			sprintf(bgerr, "Unknown element '%s'", cur->name);
			BAIL(bgerr);
		}

		cur = cur->next;
	}

	xmlFreeDoc(parsed);

	return(bgconf);
}
