/*
 * bgrot.h
 * Central header file for bgrot common stuff.
 * bgrot project
 * $Fuller: myprograms/bgrot/include/bgrot.h,v 1.21 2003/03/23 14:50:36 fullermd Exp $
 */

#ifndef _BGROT_H_
#define _BGROT_H

/* Typedefs */
typedef struct imgent IMGENT;
typedef struct ilconf ILCONF;
typedef struct imglist IMGLIST;
typedef struct index_ent INDEX_ENT;
typedef struct lindex LINDEX;
typedef struct bgrotd_conf BGROTD_CONF;


/* Prototypes */
/* Memory stuff */
BGROTD_CONF *alloc_bgrotd_conf(void);
ILCONF *alloc_ilconf(void);
IMGENT *alloc_imgent(void);
IMGLIST *alloc_imglist(void);
INDEX_ENT *alloc_index_ent(void);
LINDEX *alloc_lindex(void);
void free_bgrotd_conf(BGROTD_CONF *);
void free_ilconf(ILCONF *);
void free_imgent(IMGENT *);
void free_imgent_pp(IMGENT **, int);
void free_imglist(IMGLIST *);
void free_index_ent(INDEX_ENT *);
void free_lindex(LINDEX *);

/* Parse functions */
BGROTD_CONF *parse_conf(const char *);
LINDEX *parse_index(const char *);
IMGLIST *parse_list(const char *);

/* Randomization */
IMGENT *random_img(IMGENT ***, int *);


/* Structures */
/* imgent: One image entry in an imglist */
struct imgent
{
	char			*fname;
};

/* ilconf: An image list configuration */
struct ilconf
{
	long			interval;
	int				rvary;
	char			*display;
};

/* imglist: A single list of images */
struct imglist
{
	char			*title;
	char			*longdesc;
	float			version;
	ILCONF			*conf;
	IMGENT			**imgs;
	int				img_used;
	int				img_alloced;
};

/* index_ent: An index entry */
struct index_ent
{
	int				id;
	char			*title;
	char			*longdesc;
	char			*filename;
};

/* lindex: An index of lists */
struct lindex
{
	float			version;
	INDEX_ENT		**lists;
	int				lists_alloced;
	int				lists_used;
};

/* bgrotd_conf: The bgrotd config */
struct bgrotd_conf
{
	float			version;
	char			*basedir;
	char			*rundir;
	char			*listdir;
	LINDEX			*list_index;
	int				sel_list_id;
	IMGLIST			*sel_list;
	ILCONF			*conf_override;
};

#endif /* !_BGROT_H_ */
