/*
 * bg_main.c
 * The main entry point for bgrotd
 * bgrot project
 * $Fuller: myprograms/bgrot/server/bg_main.c,v 1.17 2004/04/20 15:31:07 fullermd Exp $
 */

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <bgrot.h>
#include <bgrotd.h>

/*
 * Global vars.
 * Note that many of these are going to be accessed cross-thread, so
 * we're going to have to slap some mutexes around them.
 */

/* Server config */
BGROTD_CONF *runcf;
pthread_mutex_t runcf_mtx=PTHREAD_MUTEX_INITIALIZER;

/* The current image */
/* ** because it'll be moving, not just changing contents */
IMGENT **cur_img;
IMGENT **nxt_img; /* Covered by same mutex */
pthread_mutex_t cur_img_mtx=PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t cur_img_cond=PTHREAD_COND_INITIALIZER;

/* Various threads */
pthread_t img_thr;

/* Debug mutex */
pthread_mutex_t debug_output_mtx=PTHREAD_MUTEX_INITIALIZER;


/* Non-thread-shared globals */
char *progname;
int debug;


/* Launch into the vortex */
int
main(int argc, char *argv[])
{
	int optch;
	char *conffile;

	progname = *argv;
	conffile = NULL;
	debug = 0;

	/* First, parse our command-line args */
	while( (optch = getopt(argc, argv, "dc:")) != -1)
		switch((char)optch)
		{
			case 'c':
				/* Config file */
				conffile = optarg;
				break;
			case 'd':
				/* Debug */
				debug++;
				break;
			default:
				usage();
				/* NOTREACHED */
		}
	
	/* Make sure we have a conffile! */
	if(conffile==NULL)
	{
		fprintf(stderr, "No config file specified!\n");
		usage(); /* Doesn't return */
	}

	/*
	 * Now parse it out.  No need to mutex the variable since we haven't
	 * started any other threads yet by this point.
	 */
	runcf = gencf(conffile);

	/*
	 * Everything from here on will be part of the 'main loop', so let's
	 * jump on into that now.
	 *
	 * Only daemonize if we're not debugging; if we are, we're staying in
	 * the foreground.
	 */
	if(debug==0)
		if( bg_daemon() < 0 )
		{
			fprintf(stderr, "Error daemonizing\n");
			exit(1);
		}

	/*
	 * We're daemonized.  Now jump in an *REALLY* start working
	 */

	/* These need to be allocated */
	cur_img = malloc(sizeof(IMGENT *));
	nxt_img = malloc(sizeof(IMGENT *));
	*cur_img = alloc_imgent();
	*nxt_img = alloc_imgent();

	/* Seed */
	/* srandomdev(3) would be nice, but it's a FreeBSDism */
	srandom((unsigned)time(NULL));

	/* Install signal handlers */
	signal(SIGTERM, emerg_exit);
	signal(SIGINT, emerg_exit);

	/* Launch the image-showing thread */
	pthread_create(&img_thr, NULL, show_images_thr, NULL);

	/* Hang around doing nothing for a long, long time */
	while(1)
		pause();

	/* Shouldn't happen */
	exit(1);
}
