/*
 * show_images_thr.c
 * The outer function of the image-showing thread.
 * bgrot project
 * $Fuller: myprograms/bgrot/server/show_images_thr.c,v 1.28 2005/04/12 05:04:45 fullermd Exp $
 */

#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <bgrot.h>
#include <bgrotd.h>

/* Pull in globals we care about */
extern BGROTD_CONF *runcf;
extern pthread_mutex_t runcf_mtx;

extern IMGENT **cur_img;
extern IMGENT **nxt_img;
extern pthread_mutex_t cur_img_mtx;
extern pthread_cond_t cur_img_cond;


/* Through the never */
void *
show_images_thr(void *dummy)
{
	IMGLIST *curlist;
	IMGENT *shownow, *shownext;
	INDEX_ENT *curlent;
	char *cffile;
	size_t mlen;
	int i, wlsize;
	int intvary, interval=0, roffset;
	char *display;
	pthread_attr_t ptattr;

	/* Init */
	curlent=NULL;
	display=NULL;

	
	/*
	 * Create an attribute.  We want our image display threads to run
	 * 'detached' (i.e., we don't have to manually pthread_join() them).
	 */
	pthread_attr_init(&ptattr);
	pthread_attr_setdetachstate(&ptattr, PTHREAD_CREATE_DETACHED);

	/* Get everything we need from the global config */
	pthread_mutex_lock(&runcf_mtx);

	/* Figure out which list file we're actually using */
	for(i=0 ; !curlent ; i++)
	{
		if(runcf->sel_list_id == (runcf->list_index->lists)[i]->id)
			curlent = (runcf->list_index->lists)[i];
	}

	/* basedir + '/' + listdir + "/" + [listfile] + '\0' */
	mlen = strlen(runcf->basedir) + 1 + strlen(runcf->listdir) + 1
			+ strlen(curlent->filename) + 1; 
	cffile = malloc(mlen);
	pthread_cleanup_push((void *)free, cffile);
	sprintf(cffile, "%s/%s/%s", runcf->basedir, runcf->listdir,
			curlent->filename);

	/* Pull out config overrides and such */
	/* XXX This should be reworked a bit; see below */
	if(runcf->conf_override)
	{
		interval = runcf->conf_override->interval;
		intvary = runcf->conf_override->rvary;
		display = strdup(runcf->conf_override->display);
	}

	/* Done with config-reading */
	pthread_mutex_unlock(&runcf_mtx);

	/* Set up cancellation stuff */
	pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

	/* Loop forever, really */
	while(1)
	{
		/* Parse out our image list file */
		curlist = parse_list(cffile);
		pthread_cleanup_push((void *)free_imglist, curlist);

		/* Check config */
		/*
		 * XXX The override handling here should be redone.  We should be
		 * able to pull new values from the curlist config every time
		 * around, but keep the overrides if they're set.  It pretty much
		 * just needs a couple flag variables, but I'm too tired to do it
		 * now.  Fix this puppy.
		 */
		if(interval==0)
		{
			interval = curlist->conf->interval;
			intvary = curlist->conf->rvary;
		}
		if(display==NULL)
			display = strdup(curlist->conf->display);

		/* Set our $DISPLAY */
		setenv("DISPLAY", display, 1);

		/* Now, our inner loop; display all the images in this list */
		D_PRINTF("Starting inner display loop...");

		/* Pick a random image to start with */
		wlsize = curlist->img_used;
		shownow = NULL;
		shownext = random_img(&curlist->imgs, &wlsize);
		pthread_cleanup_push((void *)free_imgent, shownext);

		/* Start looping */
		for( i = (curlist->img_used-1) ; i>=0 ; i-- )
		{
			pthread_t disp_thr; /* $DEITY knows how many */

			/*
			 * Rotate: 'now' is now 'next', so clean up next's old
			 * cleanup handler, find a new 'next', and create a new
			 * cleanup for it.
			 */
			pthread_cleanup_pop(0); /* free_imgent(shownext) */
			shownow = shownext;
			if(i!=0)
			{
				shownext = random_img(&curlist->imgs, &wlsize);
				pthread_cleanup_push((void *)free_imgent, shownext);
			}
			pthread_cleanup_push((void *)free_imgent, shownow);

			/* See discussion in design/display-thread */
			D_PRINTF("Spawning display of '%s'...", shownow->fname);
			pthread_create(&disp_thr, &ptattr,
					display_image_thr, (void *)(strdup(shownow->fname)));

			/*
			 * Copy shownow over into a cur_img structure, so we can
			 * refer to it elsewhere (like talking to a client).  Then
			 * stomp the cond_var for anybody who might be listening.
			 */
			pthread_mutex_lock(&cur_img_mtx);
			/* XXX We should write an imgent_dup() function */
			free((*cur_img)->fname);
			(*cur_img)->fname = strdup(shownow->fname);
			free((*nxt_img)->fname);
			(*nxt_img)->fname = strdup(shownext->fname);
			pthread_mutex_unlock(&cur_img_mtx);
			pthread_cond_broadcast(&cur_img_cond);

			/* Free it */
			pthread_cleanup_pop(1); /* free_imgent(shownow) */
			shownow=NULL;

			/* Figure out how long to sleep */
			roffset = intvary ? ((random()%(intvary*2+1))-intvary) : intvary;
			D_PRINTF("Sleeping for %d (%d + %d) seconds...",
						interval+roffset, interval, roffset);

			/* Enable cancelling while we sleep */
			pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
			pthread_testcancel();
			sleep(interval+roffset);
			pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
		}

		/* Cleanup */
		pthread_cleanup_pop(1); /* free_imglist(curlist) */
	}

	/* XXX Never happens.  We need to make a cleanup function */
	pthread_cleanup_pop(1); /* free(cffile) */

	/* Shouldn't ever really get here, I don't think... */
	exit(1);
}
