/*
 * debug.c
 * Debugging entry points
 * bgrot project
 * $Fuller: myprograms/bgrot/server/util/debug.c,v 1.4 2004/02/18 13:34:30 fullermd Exp $
 */

#include <pthread.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <bgrot.h>
#include <bgrotd.h>

/* Globals */
extern int debug;
extern pthread_mutex_t debug_output_mtx;


/* Print a line in debugging mode */
void
debug_print(int dblev, int line, const char *srcfile, const char *fmt, ...)
{
	char *msg;
	va_list ap;

	/* Don't show if we're at too low a level */
	if(dblev > debug)
		return;

	/* Prep */
	va_start(ap, fmt);

	/* Grab our output mutex so we don't stomp on ourselves */
	pthread_mutex_lock(&debug_output_mtx);

	/* Print the message */
	vasprintf(&msg, fmt, ap);
	fprintf(stderr, "DEBUG%d: %s (%s:%d)\n", dblev, msg, srcfile, line);

	/* Unlock */
	pthread_mutex_unlock(&debug_output_mtx);

	/* Cleanup */
	free(msg);
	va_end(ap);

	/* Fin */
	return;
}
