/*
 * rand_test.c
 * A program to test getting random images from a list.
 * $Fuller: myprograms/bgrot/tools/randtest/rand_test.c,v 1.3 2003/03/09 13:50:32 fullermd Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <bgrot.h>


/* Global, for convenience */
char *progname;

/* Prototype */
void usage(void);


/* Let the games begin! */
int
main(int argc, char *argv[])
{
	IMGLIST *ilist;
	IMGENT **workon;
	IMGENT **randomized;
	char *testfile;
	int i;
	int wosize;

	/* Figure out what we're doing where */
	progname=*argv;
	if(argc!=2)
		usage();
	argc--; argv++;
	testfile = *argv;

	/* Parse the file */
	printf("Parsing file '%s'...   ", testfile);
	ilist = parse_list(testfile);
	if(ilist == NULL)
	{
		fprintf(stderr, "Error in parse_list()\n");
		exit(1);
	}
	printf("Done.\n");


	/* Do our stuff */

	/* First, copy the image list */
	workon = malloc( (ilist->img_used) * sizeof(IMGENT *) );
	randomized = malloc( (ilist->img_used) * sizeof(IMGENT *) );
	for(i=0 ; i < ilist->img_used ; i++)
	{
		workon[i] = alloc_imgent();
		(workon[i])->fname = strdup((ilist->imgs)[i]->fname);
	}

	/* Seed */
	srandom((unsigned)time(NULL));

	/* Now randomize it. */
	wosize = ilist->img_used;
	for(i=0 ; i < ilist->img_used ; i++)
		randomized[i] = random_img(&workon, &wosize);

	/* Take a look at 'em */
	printf("ORIGINAL IMAGES:\n");
	for(i=0 ; i<ilist->img_used ; i++)
		printf("\to %s\n", (ilist->imgs)[i]->fname);
	
	printf("\n\nRANDOMIZED IMAGES:\n");
	for(i=0 ; i<ilist->img_used ; i++)
		printf("\tr %s\n", randomized[i]->fname);


	/* Clean up and wander off into the sunset */
	free_imgent_pp(workon, wosize);
	wosize = ilist->img_used;
	free_imgent_pp(randomized, wosize);
	free_imglist(ilist);
	exit(0);
}


void
usage(void)
{
	printf("Usage: %s <file>\n", progname);
	exit(1);
}
