/*
 * conf_test.c
 * A program to test reading the conf_test.xml image index file, using
 * libxml(2).
 * $Fuller: myprograms/bgrot/tools/xmltest/conf_test.c,v 1.5 2003/03/09 11:34:33 fullermd Exp $
 */

#include <stdlib.h>
#include <stdio.h>

#include <bgrot.h>


/* Global, for convenience */
char *progname;

/* Prototype */
void usage(void);


/* Let the games begin! */
int
main(int argc, char *argv[])
{
	BGROTD_CONF *bgconf;
	char *testfile;

	/* Figure out what we're doing where */
	progname=*argv;
	if(argc!=2)
		usage();
	argc--; argv++;
	testfile = *argv;

	/* Parse the file */
	printf("Parsing file '%s'...   ", testfile);
	bgconf = parse_conf(testfile);
	if(bgconf == NULL)
	{
		fprintf(stderr, "Error in parse_conr()\n");
		exit(1);
	}
	printf("Done.\n");


	/* Print out our results */
	printf("bgrotd config information:\n");
	printf("\tVersion: %f\n", bgconf->version);
	printf("\tBasedir: '%s'\n", bgconf->basedir);
	printf("\tRundir: '%s'\n", bgconf->rundir);
	printf("\tListdir: '%s'\n", bgconf->listdir);
	printf("\tList id to use: '%d'\n", bgconf->sel_list_id);
	if(bgconf->conf_override != NULL)
	{
		ILCONF *cono = bgconf->conf_override;

		printf("\tConfig overrides:\n");
		if(cono->interval>0)
			printf("\t\tInterval: '%li'\n", cono->interval);
		if(cono->display != NULL)
			printf("\t\tDisplay: '%s'\n", cono->display);
	}
	printf("\n\nEnd\n");


	/* Clean up and wander off into the sunset */
	free_bgrotd_conf(bgconf);
	exit(0);
}


void
usage(void)
{
	printf("Usage: %s <file>\n", progname);
	exit(1);
}
