/*
 * index_test.c
 * A program to test reading the index_test.xml image index file, using
 * libxml(2).
 * $Fuller: myprograms/bgrot/tools/xmltest/index_test.c,v 1.4 2002/11/25 11:23:26 fullermd Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <bgrot.h>


/* Global, for convenience */
char *progname;

/* Prototype */
void usage(void);


/* Let the games begin! */
int
main(int argc, char *argv[])
{
	LINDEX *list_index;
	INDEX_ENT *ient;
	char *testfile;
	char *tmpstr, *tmploc;
	int i;

	/* Figure out what we're doing where */
	progname=*argv;
	if(argc!=2)
		usage();
	argc--; argv++;
	testfile = *argv;

	/* Parse the file */
	printf("Parsing file '%s'...   ", testfile);
	list_index = parse_index(testfile);
	if(list_index == NULL)
	{
		fprintf(stderr, "Error in parse_index()\n");
		exit(1);
	}
	printf("Done.\n");


	/* Print out our results */
	printf("List index information:\n");
	printf("\tVersion: %f\n", list_index->version);
	printf("\tNumber of lists in index: '%d'\n", list_index->lists_used);
	for(i=0 ; i < list_index->lists_used ; i++)
	{
		ient = (list_index->lists)[i];
		printf("\t*** List number %d ***\n", i+1);
		printf("\t\tList id: '%d'\n", ient->id);
		printf("\t\tList name: '%s'\n", ient->title);
		tmpstr = strdup(ient->longdesc);
		while( (tmploc = strchr(tmpstr, '\n')) )
			*tmploc = ' ';
		printf("\t\tList description: '%s'\n", tmpstr);
		free(tmpstr);
		printf("\t\tList filename: '%s'\n", ient->filename);
		printf("\t*** End list number %d ***\n\n", i+1);
	}
	printf("\n\nEnd\n");


	/* Clean up and wander off into the sunset */
	free_lindex(list_index);
	exit(0);
}


void
usage(void)
{
	printf("Usage: %s <file>\n", progname);
	exit(1);
}
