/*
 * list_test.c
 * A program to test reading the list_test.xml image list file, using
 * libxml(2).
 * $Fuller: myprograms/bgrot/tools/xmltest/list_test.c,v 1.13 2003/03/09 11:34:33 fullermd Exp $
 */

#include <stdlib.h>
#include <stdio.h>

#include <bgrot.h>


/* Global, for convenience */
char *progname;

/* Prototype */
void usage(void);


/* Let the games begin! */
int
main(int argc, char *argv[])
{
	IMGLIST *ilist;
	char *testfile;
	int i;

	/* Figure out what we're doing where */
	progname=*argv;
	if(argc!=2)
		usage();
	argc--; argv++;
	testfile = *argv;

	/* Parse the file */
	printf("Parsing file '%s'...   ", testfile);
	ilist = parse_list(testfile);
	if(ilist == NULL)
	{
		fprintf(stderr, "Error in parse_list()\n");
		exit(1);
	}
	printf("Done.\n");


	/* Print out our results */
	printf("Image list information:\n");
	printf("\tVersion: %f\n", ilist->version);
	printf("\tList name: '%s'\n", ilist->title);
	printf("\tList description: '%s'\n", ilist->longdesc);
	printf("\tList configuration:\n");
	printf("\t\tInterval between images: %li\n", ilist->conf->interval);
	printf("\t\tRandom variance: %d\n",
			ilist->conf->rvary);
	printf("\t\tDisplay: '%s'\n", ilist->conf->display);
	printf("\n");
	printf("\tImages:\n");
	for( i=0 ; i < ilist->img_used ; i++ )
		printf("\t\to %s\n", (ilist->imgs)[i]->fname);
	printf("\n\nEnd\n");


	/* Clean up and wander off into the sunset */
	free_imglist(ilist);
	exit(0);
}


void
usage(void)
{
	printf("Usage: %s <file>\n", progname);
	exit(1);
}
